@@ -1,22 +1,23 @@
 /* keylist.c - Listing keys.
    Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2002, 2003 g10 Code GmbH
+   Copyright (C) 2001, 2002, 2003, 2004 g10 Code GmbH
 
    This file is part of GPGME.
  
    GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
- 
+   under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of
+   the License, or (at your option) any later version.
+   
    GPGME is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
- 
-   You should have received a copy of the GNU General Public License
-   along with GPGME; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Lesser General Public License for more details.
+   
+   You should have received a copy of the GNU Lesser General Public
+   License along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -116,40 +117,63 @@ keylist_status_handler (void *priv, gpgme_status_code_t code, char *args)
 }
 
 
-static time_t
-parse_timestamp (char *timestamp)
+static void
+set_subkey_trust_info (gpgme_subkey_t subkey, const char *src)
 {
-  if (!*timestamp)
-    return 0;
+  while (*src && !isdigit (*src))
+    {
+      switch (*src)
+	{
+	case 'e':
+	  subkey->expired = 1;
+	  break;
 
-  return (time_t) strtoul (timestamp, NULL, 10);
+	case 'r':
+	  subkey->revoked = 1;
+	  break;
+
+	case 'd':
+          /* Note that gpg 1.3 won't print that anymore but only uses
+             the capabilities field. */
+	  subkey->disabled = 1;
+	  break;
+
+	case 'i':
+	  subkey->invalid = 1;
+	  break;
+        }
+      src++;
+    }
 }
 
 
 static void
 set_mainkey_trust_info (gpgme_key_t key, const char *src)
 {
-  /* Look at letters and stop at the first digit.  */
+  /* First set the trust info of the main key (the first subkey).  */
+  set_subkey_trust_info (key->subkeys, src);
+
+  /* Now set the summarized trust info.  */
   while (*src && !isdigit (*src))
     {
       switch (*src)
 	{
 	case 'e':
-	  key->subkeys->expired = 1;
+	  key->expired = 1;
 	  break;
 
 	case 'r':
-	  key->subkeys->revoked = 1;
+	  key->revoked = 1;
 	  break;
 
 	case 'd':
           /* Note that gpg 1.3 won't print that anymore but only uses
              the capabilities field. */
-	  key->subkeys->disabled = 1;
+	  key->disabled = 1;
 	  break;
 
 	case 'i':
-	  key->subkeys->invalid = 1;
+	  key->invalid = 1;
 	  break;
         }
       src++;
@@ -197,81 +221,6 @@ set_userid_flags (gpgme_key_t key, const char *src)
 }
 
 
-static void
-set_subkey_trust_info (gpgme_subkey_t subkey, const char *src)
-{
-  /* Look at letters and stop at the first digit.  */
-  while (*src && !isdigit (*src))
-    {
-      switch (*src)
-	{
-	case 'e':
-	  subkey->expired = 1;
-	  break;
-
-	case 'r':
-	  subkey->revoked = 1;
-	  break;
-
-	case 'd':
-	  subkey->disabled = 1;
-	  break;
-
-	case 'i':
-	  subkey->invalid = 1;
-	  break;
-        }
-      src++;
-    }
-}
-
-
-static void
-set_mainkey_capability (gpgme_key_t key, const char *src)
-{
-  while (*src)
-    {
-      switch (*src)
-	{
-	case 'e':
-	  key->subkeys->can_encrypt = 1;
-	  break;
-
-	case 's':
-	  key->subkeys->can_sign = 1;
-	  break;
-
-	case 'c':
-	  key->subkeys->can_certify = 1;
-	  break;
-
-        case 'd':
-        case 'D':
-          /* Note, that this flag is also set using the key validity
-             field for backward compatibility with gpg 1.2.  We use d
-             and D, so that a future gpg version will be able to
-             disable certain subkeys. Currently it is expected that
-             gpg sets this for the primary key. */
-       	  key->subkeys->disabled = 1;
-          break;
-
-	case 'E':
-	  key->can_encrypt = 1;
-	  break;
-
-	case 'S':
-	  key->can_sign = 1;
-	  break;
-
-	case 'C':
-	  key->can_certify = 1;
-	  break;
-        }
-      src++;
-    }
-}
-
-
 static void
 set_subkey_capability (gpgme_subkey_t subkey, const char *src)
 {
@@ -290,11 +239,65 @@ set_subkey_capability (gpgme_subkey_t subkey, const char *src)
 	case 'c':
 	  subkey->can_certify = 1;
 	  break;
+
+	case 'a':
+	  subkey->can_authenticate = 1;
+	  break;
+
+	case 'd':
+	  subkey->disabled = 1;
+	  break;
         }
       src++;
     }
 }
 
+
+static void
+set_mainkey_capability (gpgme_key_t key, const char *src)
+{
+  /* First set the capabilities of the main key (the first subkey).  */
+  set_subkey_capability (key->subkeys, src);
+
+  while (*src)
+    {
+      switch (*src)
+	{
+	case 'd':
+        case 'D':
+          /* Note, that this flag is also set using the key validity
+             field for backward compatibility with gpg 1.2.  We use d
+             and D, so that a future gpg version will be able to
+             disable certain subkeys. Currently it is expected that
+             gpg sets this for the primary key. */
+       	  key->disabled = 1;
+          break;
+
+	case 'e':
+	case 'E':
+	  key->can_encrypt = 1;
+	  break;
+
+	case 's':
+	case 'S':
+	  key->can_sign = 1;
+	  break;
+
+	case 'c':
+	case 'C':
+	  key->can_certify = 1;
+	  break;
+
+	case 'a':
+	case 'A':
+	  key->can_authenticate = 1;
+	  break;
+        }
+      src++;
+    }
+}
+
+
 static void
 set_ownertrust (gpgme_key_t key, const char *src)
 {
@@ -428,6 +431,7 @@ keylist_colon_handler (void *priv, char *line)
       err = _gpgme_key_new (&key);
       if (err)
 	return err;
+      key->keylist_mode = ctx->keylist_mode;
       err = _gpgme_key_add_subkey (key, &subkey);
       if (err)
 	{
@@ -469,11 +473,11 @@ keylist_colon_handler (void *priv, char *line)
 
       /* Field 6 has the timestamp (seconds).  */
       if (fields >= 6)
-	subkey->timestamp = parse_timestamp (field[5]);
+	subkey->timestamp = _gpgme_parse_timestamp (field[5], NULL);
 
       /* Field 7 has the expiration time (seconds).  */
       if (fields >= 7)
-	subkey->expires = parse_timestamp (field[6]);
+	subkey->expires = _gpgme_parse_timestamp (field[6], NULL);
 
       /* Field 8 has the X.509 serial number.  */
       if (fields >= 8 && (rectype == RT_CRT || rectype == RT_CRS))
@@ -537,11 +541,11 @@ keylist_colon_handler (void *priv, char *line)
 
       /* Field 6 has the timestamp (seconds).  */
       if (fields >= 6)
-	subkey->timestamp = parse_timestamp (field[5]);
+	subkey->timestamp = _gpgme_parse_timestamp (field[5], NULL);
 
       /* Field 7 has the expiration time (seconds).  */
       if (fields >= 7)
-	subkey->expires = parse_timestamp (field[6]);
+	subkey->expires = _gpgme_parse_timestamp (field[6], NULL);
 
       /* Field 8 is reserved (LID).  */
       /* Field 9 has the ownertrust.  */
@@ -637,21 +641,22 @@ keylist_colon_handler (void *priv, char *line)
       
       /* Field 6 has the timestamp (seconds).  */
       if (fields >= 6)
-	keysig->timestamp = parse_timestamp (field[5]);
+	keysig->timestamp = _gpgme_parse_timestamp (field[5], NULL);
 
       /* Field 7 has the expiration time (seconds).  */
       if (fields >= 7)
-	keysig->expires = parse_timestamp (field[6]);
+	keysig->expires = _gpgme_parse_timestamp (field[6], NULL);
 
       /* Field 11 has the signature class (eg, 0x30 means revoked).  */
       if (fields >= 11)
 	if (field[10][0] && field[10][1])
 	  {
-	    int class = _gpgme_hextobyte (field[10]);
-	    if (class >= 0)
+	    int sig_class = _gpgme_hextobyte (field[10]);
+	    if (sig_class >= 0)
 	      {
-		keysig->class = class;
-		if (class == 0x30)
+		keysig->sig_class = sig_class;
+		keysig->class = keysig->sig_class;
+		if (sig_class == 0x30)
 		  keysig->revoked = 1;
 	      }
 	    if (field[10][2] == 'x')
@@ -832,8 +837,9 @@ gpgme_get_key (gpgme_ctx_t ctx, const char *fpr, gpgme_key_t *r_key,
 {
   gpgme_ctx_t listctx;
   gpgme_error_t err;
+  gpgme_key_t key;
 
-  if (!ctx || !r_key)
+  if (!ctx || !r_key || !fpr)
     return gpg_error (GPG_ERR_INV_VALUE);
   
   if (strlen (fpr) < 16)	/* We have at least a key ID.  */
@@ -849,6 +855,21 @@ gpgme_get_key (gpgme_ctx_t ctx, const char *fpr, gpgme_key_t *r_key,
   err = gpgme_op_keylist_start (listctx, fpr, secret);
   if (!err)
     err = gpgme_op_keylist_next (listctx, r_key);
+  if (!err)
+    {
+      err = gpgme_op_keylist_next (listctx, &key);
+      if (gpgme_err_code (err) == GPG_ERR_EOF)
+	err = gpg_error (GPG_ERR_NO_ERROR);
+      else
+	{
+	  if (!err)
+	    {
+	      gpgme_key_unref (key);
+	      err = gpg_error (GPG_ERR_AMBIGUOUS_NAME);
+	    }
+	  gpgme_key_unref (*r_key);
+	}
+    }
   gpgme_release (listctx);
   return err;
 }
