@@ -1,6 +1,6 @@
 /* wait-global.c 
    Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2002, 2003, 2004, 2005 g10 Code GmbH
+   Copyright (C) 2001, 2002, 2003, 2004 g10 Code GmbH
  
    This file is part of GPGME.
  
@@ -32,7 +32,7 @@
 #include "util.h"
 #include "context.h"
 #include "wait.h"
-#include "priv-io.h"
+#include "io.h"
 
 /* The global event loop is used for all asynchronous operations
    (except key listing) for which no user I/O callbacks are specified.
@@ -310,7 +310,7 @@ gpgme_wait (gpgme_ctx_t ctx, gpgme_error_t *status, int hang)
 	      ictx = item->ctx;
 	      assert (ictx);
 
-	      err = _gpgme_run_io_cb (&fdt.fds[i], 0);
+	      err = item->handler (item->handler_value, fdt.fds[i].fd);
 	      if (err)
 		{
 		  /* An error occured.  Close all fds in this context,
@@ -322,11 +322,6 @@ gpgme_wait (gpgme_ctx_t ctx, gpgme_error_t *status, int hang)
 		      _gpgme_io_close (ictx->fdt.fds[idx].fd);
 		  _gpgme_engine_io_event (ictx->engine, GPGME_EVENT_DONE,
 					  &err);
-
-		  /* Break out of the loop, and retry the select()
-		     from scratch, because now all fds should be
-		     gone.  */
-		  break;
 		}
 	    }
 	}
@@ -334,30 +329,15 @@ gpgme_wait (gpgme_ctx_t ctx, gpgme_error_t *status, int hang)
 
       /* Now some contexts might have finished successfully.  */
       LOCK (ctx_list_lock);
-    retry:
       for (li = ctx_active_list; li; li = li->next)
 	{
-	  gpgme_ctx_t actx = li->ctx;
-
-	  for (i = 0; i < actx->fdt.size; i++)
-	    if (actx->fdt.fds[i].fd != -1)
+	  for (i = 0; i < ctx->fdt.size; i++)
+	    if (ctx->fdt.fds[i].fd != -1)
 	      break;
-	  if (i == actx->fdt.size)
+	  if (i == ctx->fdt.size)
 	    {
 	      gpgme_error_t err = 0;
-
-	      /* FIXME: This does not perform too well.  We have to
-		 release the lock because the I/O event handler
-		 acquires it to remove the context from the active
-		 list.  Two alternative strategies are worth
-		 considering: Either implement the DONE event handler
-		 here in a lock-free manner, or save a list of all
-		 contexts to be released and call the DONE events
-		 afterwards.  */
-	      UNLOCK (ctx_list_lock);
-	      _gpgme_engine_io_event (actx->engine, GPGME_EVENT_DONE, &err);
-	      LOCK (ctx_list_lock);
-	      goto retry;
+	      _gpgme_engine_io_event (ctx->engine, GPGME_EVENT_DONE, &err);
 	    }
 	}
       UNLOCK (ctx_list_lock);
@@ -370,12 +350,6 @@ gpgme_wait (gpgme_ctx_t ctx, gpgme_error_t *status, int hang)
 	    ctx = dctx;
 	    hang = 0;
 	  }
-	else if (!hang)
-	  {
-	    ctx = NULL;
-	    if (status)
-	      *status = 0;
-	  }
       }
     }
   while (hang);
