@@ -1,22 +1,29 @@
 /* t-verify.c - Regression test.
    Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2002, 2003 g10 Code GmbH
+   Copyright (C) 2001, 2002, 2003, 2004 g10 Code GmbH
 
    This file is part of GPGME.
  
    GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
- 
+   under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of
+   the License, or (at your option) any later version.
+   
    GPGME is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
- 
-   You should have received a copy of the GNU General Public License
-   along with GPGME; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Lesser General Public License for more details.
+   
+   You should have received a copy of the GNU Lesser General Public
+   License along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
+
+/* We need to include config.h so that we know whether we are building
+   with large file system (LFS) support. */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -71,7 +78,7 @@ static const char test_sig2[] =
 
 
 static void
-check_result (gpgme_verify_result_t result, int summary, char *fpr,
+check_result (gpgme_verify_result_t result, unsigned int summary, char *fpr,
 	      gpgme_error_t status, int notation)
 {
   gpgme_signature_t sig;
@@ -85,8 +92,9 @@ check_result (gpgme_verify_result_t result, int summary, char *fpr,
     }
   if (sig->summary != summary)
     {
-      fprintf (stderr, "%s:%i: Unexpected signature summary: 0x%x\n",
-	       __FILE__, __LINE__, sig->summary);
+      fprintf (stderr, "%s:%i: Unexpected signature summary: "
+               "want=0x%x have=0x%x\n",
+	       __FILE__, __LINE__, summary, sig->summary);
       exit (1);
     }
   if (strcmp (sig->fpr, fpr))
@@ -103,24 +111,56 @@ check_result (gpgme_verify_result_t result, int summary, char *fpr,
     }
   if (notation)
     {
-      if (!sig->notations
-	  || strcmp (sig->notations->name, "bar")
-	  || strcmp (sig->notations->value, "\xc3\xb6\xc3\xa4\xc3\xbc\xc3\x9f"
-		     " das waren Umlaute und jetzt ein prozent%-Zeichen")
-	  || !sig->notations->next
-	  || strcmp (sig->notations->next->name, "foobar.1")
-	  || strcmp (sig->notations->next->value,
-		     "this is a notation data with 2 lines")
-	  || !sig->notations->next->next
-	  || sig->notations->next->next->name != NULL
-	  || strcmp (sig->notations->next->next->value,
-		     "http://www.gu.org/policy/")
-	  || sig->notations->next->next->next)
-	{
-	  fprintf (stderr, "%s:%i: Unexpected notation data\n",
-		   __FILE__, __LINE__);
-	  exit (1);
-	}
+      static struct {
+        const char *name;
+        const char *value;
+        int seen;
+      } expected_notations[] = {
+        { "bar",
+	  "\xc3\xb6\xc3\xa4\xc3\xbc\xc3\x9f"
+          " das waren Umlaute und jetzt ein prozent%-Zeichen" },
+        { "foobar.1",
+	  "this is a notation data with 2 lines" },
+        { NULL, 
+	  "http://www.gu.org/policy/" }
+      };
+      int i;
+      gpgme_sig_notation_t r;
+
+      for (i=0; i < DIM(expected_notations); i++ )
+        expected_notations[i].seen = 0;
+
+      for (r = sig->notations; r; r = r->next)
+        {
+          int any = 0;
+          for (i=0; i < DIM(expected_notations); i++)
+            {
+              if ( ((r->name && expected_notations[i].name
+                     && !strcmp (r->name, expected_notations[i].name))
+                    || (!r->name && !expected_notations[i].name))
+                   && r->value
+                   && !strcmp (r->value, expected_notations[i].value))
+                {
+                  expected_notations[i].seen++;
+                  any++;
+                }
+            }
+          if (!any)
+            {
+              fprintf (stderr, "%s:%i: Unexpected notation data\n",
+                       __FILE__, __LINE__);
+              exit (1);
+            }
+        }
+      for (i=0; i < DIM(expected_notations); i++ )
+        {
+          if (expected_notations[i].seen != 1)
+            {
+              fprintf (stderr, "%s:%i: Missing or duplicate notation data\n",
+                       __FILE__, __LINE__);
+              exit (1);
+            }
+        }
     }
   if (sig->wrong_key_usage)
     {
@@ -151,6 +191,8 @@ main (int argc, char *argv[])
   gpgme_data_t sig, text;
   gpgme_verify_result_t result;
 
+  init_gpgme (GPGME_PROTOCOL_OpenPGP);
+
   err = gpgme_new (&ctx);
   fail_if_err (err);
 
